/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * Models a real-world bucket. A bucket is characteries by its
 * capacity (measured in liters) and its quantity - the amount of
 * water the bucket contain.
 *
 * @author Shmulik London
 */
public class Bucket {

    // The capacity of the bucket
    private int capacity;

    // The amount of liquid in the bucket
    private int quantity;

    // The name of the bucket
    private String name;

    // Holds the number of created buckets
    private static int numberOfBuckets;

    // Holds the number of buckets that has a non default name
    private static int namedBuckets;

    /**
     * Constructs a new Bucket with a given capacity.
     * @param capacity The capacity of the bucket in liters.
     * @exception IllegalArgumentException An attempt to
     * create a bucket with non-positive capacity.
     */
    public Bucket(int capacity) {
        this(capacity,null);
    }

    /**
     * Constructs a new Bucket with a given capacity
     * and name.
     * @param capacity The capacity of the bucket in liters.
     * @param name The name of the bucket.
     * @exception IllegalArgumentException An attempt to
     * create a bucket with non-positive capacity.
     */
    public Bucket(int capacity, String name) {
        if (capacity <= 0) {
            throw new IllegalArgumentException(
                "Bucket capacity must be a positive number!");
        }
        numberOfBuckets++;
        this.capacity = capacity;
        if (name == null) {
            this.name = String.valueOf((char)('A'+numberOfBuckets-namedBuckets-1));
        } else {
            namedBuckets++;
            this.name = name;
        }
        BucketsVisualization.createBucketFigure(this);
    }

    /**
     * Returns the amount of liquid in the bucket in
     * Liters.
     */
    public final int getQuantity() {
        return quantity;
    }

    /**
     * Returns the capacity of this bucket.
     */
    public final int getCapacity() {
        return capacity;
    }

    /**
     * Returns the name of this bucket.
     */
    public final String getName() {
        return name;
    }

    /**
     * Return a string describing this bucket.
     */
    public String toString() {
        return "Bucket "+getName();
    }

    /**
     * Empties the bucket.
     */
    public void empty() {
        BucketsVisualization.displayBucketEmptying(this);
        quantity = 0;
    }

    /**
     * Fills the bucket.
     */
    public void fill() {
        BucketsVisualization.displayBucketFilling(this);
        quantity = capacity;
    }

    /**
     * Pour water from this bucket to a target bucket.
     * This method pours as much water as possible, if the
     * target bucket has become full water may be left in
     * this bucket.
     * @param bucket The target bucket.
     * @exception BucketException The target bucket is the
     * same bucket as the source bucket (this bucket).
     */
    public void pourInto(Bucket bucket) {
        if (bucket == this) {
            throw new BucketException(
                "You cannot pour a bucket into itself!");
        }
        BucketsVisualization.displayBucketPuring(this,bucket);
        int reserve = bucket.capacity - bucket.quantity;
        int amount = Math.min(quantity,reserve);
        quantity -= amount;
        bucket.quantity += amount;
    }

}

/**
 * An exception indicating illegal operations on buckets.
 */
class BucketException extends RuntimeException {

    /**
     * Creates a new BucketException.
     */
    public BucketException() {
    }

    /**
     * Creates a new BucketException.
     * @param reason The reason for the exception.
     */
    public BucketException(String reason) {
        super(reason);
    }
}