/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;

/**
 * A figure of a bucket. Used for the display of bucket
 * problems.
 *
 * @author Shmulik London
 */
public class BucketFigure extends Component {

    private static final int DEFAULT_WIDTH = 100;
    private static final int DEFAULT_HEIGHT_PER_LITER = 30;

    private int heightPerLiter = DEFAULT_HEIGHT_PER_LITER;

    // The bucket this component display
    private Bucket bucket;

    // The color of the liquid in the bucket
    private Color liquidColor = Color.blue;

    // The thickness of the frames line
    private int frameThickness = 3;

    private Insets margins =
        new Insets(frameThickness+5, // top
                   frameThickness,   // left
                   frameThickness+20,   // bottom
                   frameThickness);  // right

    private boolean quantityIsChanging;
    private float quantity;

    /**
     * Constructs a new BucketFigure.
     */
    public BucketFigure(Bucket bucket) {
        this.bucket = bucket;
        setForeground(Color.black);
        quantity = bucket.getQuantity();
    }

    /**
     * Paints this component.
     */
    public void paint(Graphics g) {
        paintBucketFrame(g);
        paintBucketContent(g);
        paintBucketScale(g);
        paintLabel(g);
    }

    /**
     * Paints the frame of the bucket.
     */
    protected void paintBucketFrame(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;
        int bottom = h-margins.bottom;
        g.setColor(getForeground());
        for (int i=0; i<frameThickness; i++) {
            g.drawLine(i,0,i,bottom);
            g.drawLine(0,bottom-i,w-1,bottom-i);
            g.drawLine(w-1-i,bottom,w-1-i,0);
        }
    }

    /**
     * Paints the content of the bucket.
     */
    protected void paintBucketContent(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;
        int left = frameThickness;
        int top = h-margins.bottom-liquidHieght();
        int width = w-margins.left-margins.right;
        int height = liquidHieght()-1;
        g.setColor(liquidColor);
        g.fillRect(left,top,width,height);
    }

    /**
     * Paints a scale on the bucket.
     */
    protected void paintBucketScale(Graphics g) {
        int capacity = (int)bucket.getCapacity();
        int y = getSize().height-margins.bottom;
        g.setColor(Color.yellow);
        int left = frameThickness;
        int right = getSize().width-frameThickness-1;
        int smallTick = heightPerLiter/5;
        for (int i=1; i<=capacity; i++) {
            for (int j=1; j<5; j++) {
                int yy = y-j*smallTick;
                g.drawLine(left,yy,left+10,yy);
            }
            y-=heightPerLiter;
            g.drawLine(left,y,right,y);
            g.drawString(String.valueOf(i)+"L",left+20,y+15);
        }
    }

    /**
     * Paints the label of this bucket.
     */
    protected void paintLabel(Graphics g) {
        String label = bucket.getName();
        g.setColor(Color.black);
        g.setFont(new Font("Dialog",Font.BOLD,14));
        FontMetrics metrics = g.getFontMetrics();
        int left = (getSize().width-metrics.stringWidth(label))/2;
        int baseline = getSize().height-5;
        g.drawString(label,left,baseline);
    }

    // computes the height of the liquid content in pixels
    private int liquidHieght() {
        if (quantityIsChanging) {
            return (int)(quantity*heightPerLiter);
        }
        return bucket.getQuantity()*heightPerLiter;
    }

    /**
     * Allows the quantity of the figure to change in non integral
     * steps.
     */
    protected void setEnableChangeQuantity(boolean flag) {
        quantityIsChanging = flag;
    }

    /**
     * Sets the quantity of liquid displayed by this figure.
     */
    public void setQuantity(float quantity) {
        this.quantity = quantity;
    }

    /**
     * Returns the quantity of liquid displayed by this figure.
     */
    public float getQuantity() {
        return quantity;
    }

    /**
     * Returns the preferred size of this component.
     */
    public Dimension getPreferredSize() {
        int width = DEFAULT_WIDTH;
        int height = bucket.getCapacity()*heightPerLiter+margins.top+margins.bottom;
        return new Dimension(width,height);
    }

    /**
     * Return the minimal size of this component.
     */
    public Dimension getMinimalSize() {
        return getPreferredSize();
    }

}