/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.util.*;
import java.awt.event.*;

import jclip.*;

/**
 * @author Shmulik London
 */
public class BucketsVisualization {

    static final long PREFERRED_FRAME_DELAY = 100;

    // The frame on which the buckets are displayed
    private static BucketsDisplayFrame display;

    // A frame reporting the action taking place
    private static BucketsActionReporter reporter;

    // A mapping between buckets and their representative
    // components
    private static Hashtable bucketFigures;

    static {
        display = new BucketsDisplayFrame();
        reporter = new BucketsActionReporter();
        bucketFigures = new Hashtable();
    }

    /**
     * Creates a graphical component that represent a given
     * bucket.
     */
    public static void createBucketFigure(Bucket bucket) {
        if (bucketFigures.isEmpty()) {
            display.setLocation(100,50);
            display.setVisible(true);
            reporter.setLocation(
                display.getLocation().x+display.getSize().width+20,
                display.getLocation().y-20);
            reporter.setVisible(true);
        }
        BucketFigure figure = new BucketFigure(bucket);
        display.addBucket(figure);
        reporter.reportBucketCreation(bucket);
        bucketFigures.put(bucket,figure);
    }

    /**
     * Display the action of filling a given bucket.
     */
    public static void displayBucketFilling(Bucket bucket) {
        reporter.reportBucketFilling(bucket);
        BucketFigure bucketFigure =
            (BucketFigure) bucketFigures.get(bucket);
        AnimationClip clip =
            new ChangeQuantityClip(bucketFigure,
                bucket.getQuantity(),bucket.getCapacity(),40);
        clip.play(true);
    }

    /**
     * Displays the action of emptying a given bucket.
     */
    public static void displayBucketEmptying(Bucket bucket) {
        reporter.reportBucketEmptying(bucket);
        BucketFigure bucketFigure =
            (BucketFigure) bucketFigures.get(bucket);
        AnimationClip clip =
            new ChangeQuantityClip(bucketFigure,
                bucket.getQuantity(),0,40);
        clip.play(true);
    }

    /**
     * Displays the action of puring the content of one bucket
     * to another.
     */
    public static void displayBucketPuring(Bucket source, Bucket target) {
        reporter.reportBucketPuring(source,target);

        if (source.getQuantity()==0 ||
            target.getQuantity()==target.getCapacity()) {
            return;
        }
        BucketFigure sourceFigure =
            (BucketFigure) bucketFigures.get(source);
        BucketFigure targetFigure =
            (BucketFigure) bucketFigures.get(target);

        int reserve = target.getCapacity() - target.getQuantity();
        int amount = Math.min(source.getQuantity(),reserve);

        AnimationClip clip1 =
            new ChangeQuantityClip(sourceFigure,
                source.getQuantity(),source.getQuantity()-amount,40);

        AnimationClip clip2 =
            new ChangeQuantityClip(targetFigure,
                target.getQuantity(),target.getQuantity()+amount,40);

        AnimationClip clip = new AnimationClipUnion(clip1,clip2);
        clip.setPreferredFrameDelay(PREFERRED_FRAME_DELAY);
        clip.play(true);
    }
}

class BucketsDisplayFrame extends Frame {

    Panel display;

    public BucketsDisplayFrame() {
        setSize(500,400);
        add(display = new BucketDisplayPanel(),BorderLayout.CENTER);
        Panel bottomPanel = new Panel();
        Button exitButton = new Button("Exit");
        bottomPanel.add(exitButton);
        add(bottomPanel, BorderLayout.SOUTH);
        exitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                System.exit(0);
            }
        });
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent event) {
                System.exit(0);
            }
        });

    }

    public void addBucket(BucketFigure bucket) {
        display.add(bucket);
        display.validate();
    }
}

class BucketDisplayPanel extends DoubleBufferedPanel {

    public BucketDisplayPanel() {
        setLayout(new GroundLayout());
    }

}

class ChangeQuantityClip extends AnimationClip {

    BucketFigure bucket;
    private float q1,q2;
    private float step;

    public ChangeQuantityClip(BucketFigure bucket, float q1, float q2, int n) {
        this.bucket = bucket;
        this.q1 = q1;
        this.q2 = q2;
        this.step = (q2-q1)/n;
        setPreferredFrameDelay(BucketsVisualization.PREFERRED_FRAME_DELAY);
    }

    public void init() {
        bucket.setEnableChangeQuantity(true);
        bucket.setQuantity(q1);
    }

    public void update() {
        float q = bucket.getQuantity();
        if ((step>0 && q>=q2)|| (step<0 && q<=q2)) {
            bucket.setQuantity(q2);
            bucket.repaint();
            endClip();
            return;
        }
        q+=step;
        bucket.setQuantity(q);
        bucket.repaint();
    }
}

class BucketsActionReporter extends Frame {

    TextArea text;

    public BucketsActionReporter() {
        setSize(400,300);
        setBackground(new Color(255,255,204));
        text = new TextArea();
        text.setBackground(new Color(255,255,204));
        text.setForeground(new Color(0,0,120));
        text.setFont(new Font("Dialog",Font.BOLD,16));
        add(text,"Center");
    }

    public void reportBucketCreation(Bucket bucket) {
        report("Created "+bucket);
    }

    public void reportBucketFilling(Bucket bucket) {
        report("Filling "+bucket);
    }

    public void reportBucketEmptying(Bucket bucket) {
        report("Emptying "+bucket);
    }

    public void reportBucketPuring(Bucket source, Bucket target) {
        report("Puring from "+source+" to "+target);
    }

    protected void report(String description) {
        text.appendText(description+"\n");
    }

}