import java.awt.*;

/**
 * A component that displays graphically a chess-board
 */
public class ChessBoardComponent extends Container {

    private Dimension cacheSize;

    private static final Color BLACK_CELLS_DEFAULT_COLOR = new Color(153,102,51);
    private static final Color WHITE_CELLS_DEFAULT_COLOR = new Color(255,255,102);
    private static final Color FRAME_DEFAULT_COLOR = new Color(102,51,51);

    private Color blackCellsColor=BLACK_CELLS_DEFAULT_COLOR;
    private Color whiteCellsColor=WHITE_CELLS_DEFAULT_COLOR;
    private Color frameColor = FRAME_DEFAULT_COLOR;
    private Color labelsColor = Color.yellow;

    private static final int PREFERRED_CELL_SIZE = 35;
    private static final int MINIMAL_CELL_SIZE = 10;

    // The font used for the labels of the rows and columns of the board
    private Font font = new Font("Dialog", Font.BOLD, 14);

    // When set to true the labels of the columns and rows are displayed.
    private boolean paintLabels = true;


    // The number of rows and columns of the chess-board
    private int rows, columns;

    private Insets insets;

    /**
     * Constructs a chessboard component.
     */
    public ChessBoardComponent(int numberOfColumns, int numberOfRows) {
        this.rows = numberOfRows;
        this.columns = numberOfColumns;
        setBackground(FRAME_DEFAULT_COLOR);
    }

    /**
     * Constructs a standard 8x8 chessboard component.
     */
    public ChessBoardComponent() {
        this(8,8);
    }

    /**
     * Sets the color used for the black cells of the board.
     */
    public void setBlackCellsColor(Color color) {
        blackCellsColor = color;
        repaint();
    }

    /**
     * Returns the color used for the black cells of the board.
     */
    public Color getBlackCellsColor() {
        return blackCellsColor;
    }

    /**
     * Sets the color used for the white cells of the board.
     */
    public void setWhiteCellsColor(Color color) {
        whiteCellsColor = color;
        repaint();
    }

    /**
     * Returns the color used for the black cells of the board.
     */
    public Color getWhiteCellsColor() {
        return whiteCellsColor;
    }

    /**
     * Sets the color used to paint the labels of the columns and rows.
     * @param color The new color for the labels.
     */
    public void setLabelsColor(Color color) {
        labelsColor = color;
        if (paintLabels) {
            repaint();
        }
    }

    /**
     * Returns the color used to paint the columns and rows.
     */
    public Color getLabelsColor() {
        return labelsColor;
    }

    /**
     * Sets the font used to paint the labels of the columns and rows.
     * @param font The new font for the labels.
     */
    public void setLabelsFont(Font font) {
        this.font = font;
        if (paintLabels) {
            repaint();
        }
    }

    /**
     * Returns the font used to paint the columns and rows.
     */
    public Font getLabelsFont() {
        return font;
    }

    /**
     * Paints this component.
     */
    public void paint(Graphics g) {
        if (cacheSize==null || !cacheSize.equals(getSize())) {
            cacheSize = getSize();
            insets = getPreferredInsets();
        }
        paintFrame(g);
        paintCells(g);
    }

    /**
     * Paints the cells of the chessboard.
     */
    protected void paintCells(Graphics g) {
        int x = insets.left;
        int y = insets.top;
        int width = getSize().width-insets.left-insets.right;
        int height = getSize().height-insets.top-insets.bottom;
        int dx = width/columns;
        int dy = height/rows;

        g.setColor(blackCellsColor);
        g.fillRect(x,y,width,height);
        g.setColor(whiteCellsColor);
        for (int i=0; i<rows; i++) {
            for (int j=0; j<columns; j++){
                if ((i+j)%2==0) {
                    g.fillRect(x+i*dx,y+j*dy,dx,dy);
                }
            }
        }
    }

    /**
     * Enable/disables the painting of the labels of the columns and rows.
     */
    public void setEnabledLabelsPainting(boolean flag) {
        paintLabels = flag;
        insets = getPreferredInsets();
        repaint();
    }

    /**
     * Checks if the labels of the rows and columns are painted.
     */
    public boolean isEnabledLabelsPainting() {
        return paintLabels;
    }

    /**
     * Paints the frame of the board.
     */
    protected void paintFrame(Graphics g) {
        int x = insets.left;
        int y = insets.top;
        int width = getSize().width-insets.left-insets.right;
        int height = getSize().height-insets.top-insets.bottom;
        g.setColor(getBackground());
        g.fillRect(0, 0, getSize().width, getSize().height);
        if (paintLabels) {
            paintLabels(g);
        }
    }

    /**
     * Paints the labels of the columns and rows of this board.
     */
    protected void paintLabels(Graphics g) {
        g.setColor(labelsColor);
        g.setFont(font);
        FontMetrics metrics = getFontMetrics(font);
        for (int i=0; i<rows; i++) {
            Rectangle cell = getCellBounderies(0,i);
            String label = getRowLabel(i);
            int x=(insets.left-metrics.stringWidth(label))/2;
            int y=cell.y+cell.height/2+5;
            g.drawString(label,x,y);
        }
        for (int i=0; i<columns; i++) {
            Rectangle cell = getCellBounderies(i,rows-1);
            int x=cell.x+cell.width/2-5;
            int y=getSize().height-insets.bottom/2+metrics.getAscent()/2;
            String label = getColumnLabel(i);
            g.drawString(label,x,y);
        }
    }

    /**
     * Returns the label that should be used to name a given row.
     * @param row The index of the row, when the top row is numbered 0.
     */
    protected String getRowLabel(int row) {
        return String.valueOf(rows-row);
    }

    /**
     * Returns the label that should be used to name a given column.
     * @param column The index of the column, wheh the left column is numbered 0.
     */
    protected String getColumnLabel(int column) {
        return String.valueOf((char)('A'+column));
    }

    /**
     * Returns the preferred insets to be used for the component.
     * When calling <TT>setEnabledLabelsPainting()</TT> sets the insets
     * according to the value suggested by this method.
     */
    protected Insets getPreferredInsets() {
        int size = paintLabels ? 20 : 5;
        if (getSize().width==0) {
            return new Insets(size, size, size, size);
        }
        int hSize = size;
        int vSize = size;

        int width = getSize().width-2*size;
        int height = getSize().height-2*size;
        int hReminder = width%columns;
        int vReminder = height%rows;
        hSize += hReminder/2;
        vSize += vReminder/2;

        // IMPROVE
        return new Insets(vSize, hSize, vSize, hSize);
    }

    /**
     * Returns the coordinates of the cell containing the given point.
     * @param x,y The coordinate of the point.
     * @return The coordinates of the cell containing the point or null
     * if there is no such cell.
     */
    public Point getCell(int x, int y) {
        int left = insets.left;
        int top = insets.top;
        int width = getSize().width-insets.left-insets.right;
        int height = getSize().height-insets.top-insets.bottom;
        int dx = width/columns;
        int dy = height/rows;

        int col = (int)((double)(x-left)/dx);
        int row = (int)((double)(y-top)/dy);
        if (col<0 || col>=columns || row<0 || row>=rows) {
            return null;
        }
        return new Point(col,row);
    }

    /**
     * Returns the bounderies of a given cell.
     * @param col, row The coordinates of the cell.
     */
    public Rectangle getCellBounderies(int col, int row) {
        int left = insets.left;
        int top = insets.top;
        int width = getSize().width-insets.left-insets.right;
        int height = getSize().height-insets.top-insets.bottom;
        int dx = width/columns;
        int dy = height/rows;

        return new Rectangle(left+col*dx,top+row*dy,dx,dy);
    }

    /**
     * Returns the center point of a given cell.
     * @param col, row The coordinates of the cell.
     */
    public Point getCellCenter(int col, int row) {
        int left = insets.left;
        int top = insets.top;
        int width = getSize().width-insets.left-insets.right;
        int height = getSize().height-insets.top-insets.bottom;
        int dx = width/columns;
        int dy = height/rows;

        return new Point(left+col*dx+dx/2,top+row*dy+dy/2);
    }

    /**
     * Returns the center point of a given cell.
     * @param cell The given cell.
     */
    public Point getCellCenter(Point cell) {
        return getCellCenter(cell.x, cell.y);
    }

    /**
     * Returns the preferred size of this component.
     */
    public Dimension getPreferredSize() {
        Insets insets = getPreferredInsets();
        int width = insets.left+PREFERRED_CELL_SIZE*columns+insets.right;
        int height = insets.top+PREFERRED_CELL_SIZE*rows+insets.bottom;
        return new Dimension(width, height);
    }

    /**
     * Returns the minimal size of this component.
     */
    public Dimension getMinimalSize() {
        int width = 2+MINIMAL_CELL_SIZE*columns+2;
        int height = 2+MINIMAL_CELL_SIZE*rows+2;
        return new Dimension(width, height);
    }

}