/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package logo;

import java.util.*;
import java.io.Serializable;

/**
 * A turtle of LOGO.
 * @author Shmulik London
 */
public class Turtle {

    // The location of the turtle in hi-resolution coordinates
    private double x,y;

    // The direction of the turtle in degrees
    private double alpha = 90;

    // True iff the tail of the turtle is down
    private boolean tailDown;

    // The image of this turtle
    private TurtleImage image;

    // The delay after each operation of the turtle
    private int delay = 10;
    
    // A memory of the homes of the turtle
    private Stack homes = new Stack();

    /**
     * Constructs a new turtle. <BR>
     * Note: In this version, the construction of the first
     * turtle has a side effect of opening a new window and
     * placing the turtle onto it. Any additional created
     * turtles will be placed on the same window.
     */
    public Turtle() {
        image = new TurtleImage();
        image.setAngle(degreesToRadians(alpha));
    }

    /**
     * Advances the turtle forwards by a given number of
     * units. The turtle advances in the direction it is
     * currently looking at.
     * @param units The number of units the turtle should
     * advance by.
     */
    public void moveForward(double units) {
        double dx = units*Math.cos(degreesToRadians(alpha));
        double dy = -units*Math.sin(degreesToRadians(alpha));
        x+=dx;
        y+=dy;
        image.moveTo((int)x,(int)y,tailDown);
        delay();
    }

    /**
     * Moves the turtle backwards by a given number of units.
     * @param units The number of units the turtle shuold
     * move backwards by.
     * @see #moveForward(double)
     */
    public void moveBackward(double units) {
        moveForward(-units);
    }

    /**
     * Turns the turtle counter-clockwise. The turtle
     * changes the direction it is looking at by the given
     * number of degrees.
     * @param degrees The angle by which the turtle should
     * turn
     */
    public void turnLeft(int degrees) {
        alpha+=degrees;
        image.setAngle(degreesToRadians(alpha));
        delay();
    }

    /**
     * Turns the turtle clockwise. The turtle changes the
     * direction it is looking at by the given number of
     * degrees.
     * @param degrees The angle by which the turtle should
     * turn
     */
    public void turnRight(int degrees) {
        turnLeft(-degrees);
    }

    /**
     * Lowers the tail of the turtle. Consequent movements
     * of the turtle will leave behind it a line.
     */
    public void tailDown() {
        tailDown = true;
    }

    /**
     * Raises the tail of the turtle. Consequent movements
     * of the turtle will not leave any mark behind it.
     */
    public void tailUp() {
        tailDown = false;
    }

    /**
     * Hides the turtle.
     */
    public void hide() {
        image.setVisible(false);
    }

    /**
     * Shows the turtle.
     */
    public void show() {
        image.setVisible(true);
    }
    
    /**
     * Moves the turtle to a new location.
     * @param x,y The coordinates of the new location.
     */
    public void moveTo(double x, double y) {
        this.x = x;
        this.y = y;
        image.moveTo((int)x,(int)y,tailDown);
    }
    
    /**
     * Sets the angle of the turtle.
     * @param angle The new angle.
     */
    public void setAngle(double angle) {
        this.alpha = angle;
        image.setAngle(degreesToRadians(alpha));
    }
    
    /**
     * Returns the angle of the turtle.
     * @return The angle of the turtle.
     */
    public double getAngle() {
        return alpha;
    }
    
    /**
     * Return the x coordinate of the turtle.
     */
    public double getX() {
        return x;
    }
    
    /**
     * Return the y coordinate of the turtle.
     */
    public double getY() {
        return y;
    }

    
    /**
     * Sets the amount of time (in seconds) the turtle
     * rests after doing an action.
     * @param delay The delay (resting period) in seconds.
     */
    public void setDelay(int delay) {
        this.delay = delay;
    }

    // Delay the execution of consequent instructions
    private void delay() {
        if (delay==0) {
            return;
        }
        try {
            Thread.sleep(delay);
        } catch (InterruptedException ie) {
        }
    }

    // Converts degrees to radians
    private double degreesToRadians(double degrees) {
        return (Math.PI*degrees/180);
    }
    
    /**
     * Records the location of the turtle. 
     */
    public void settle() {
        State state = new State(x,y,alpha,tailDown);
        System.out.println("storing: x="+x+" y="+y+"a="+alpha);
        homes.push(state);
    }
    
    /**
     * Returns the turtle to the last recorded position (location and direction)
     * of the turtle.
     */
    public void home() {
        if (homes.isEmpty()) {
            return;
        }
        State state = (State)homes.pop();
        //tailUp();
        moveTo(state.x, state.y);
        setAngle(state.angle);
        tailDown = state.tailDown;
        System.out.println("loading: x="+x+" y="+y+"a="+alpha);
    }
    
    class State implements Serializable {
        
        private double x,y;
        private double angle;
        private boolean tailDown;
        
        public State(double x, double y, double angle, boolean tailDown) {
            this.x = x;
            this.y = y;
            this.angle = angle;
            this.tailDown = tailDown;
        }
    }
}

