/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.util.*;
import jclip.*;
import clips.motion.*;

public class HanoiVisualizationPanel extends DoubleBufferedPanel {

    private static final Color[] palette = {
        Color.red, Color.blue, Color.green, Color.yellow, Color.cyan
    };
        
    RobotArmFigure robot;
    Vector piles;
    Hashtable mapDiskToIcon = new Hashtable();
    int leftMargin=100;
    int rightMargin=200;

    private int ground;

    public HanoiVisualizationPanel() {
        piles = new Vector();
    }

    public void init() {
        setLayout(null);
        ground = getSize().height-100;
        robot = new RobotArmFigure(getSize().height*2/3,30);
        int x = getSize().width-100;
        int dx = -1;
        int y = ground;
        robot.setLocation(x,y);
        robot.setHeadLocation(new Point(x+dx*200,y-200));
    }
    
    protected int getPileHorizontalPosition(int pile) {
        int n = piles.size();
        int width = getSize().width-leftMargin-rightMargin;
        int pileWidth = width/n;
        int x = leftMargin+pileWidth*pile+pileWidth/2;
        return x;
    }

    public void visualizePileCreation(Pile pile) {
        piles.addElement(pile);
    }
    
    private int totalDisks;

    private Color chooseColor(Disk disk) {
        Color c = palette[totalDisks % palette.length];
        totalDisks++;
        return c;
    }
    
    public void visualizeAddDisk(Pile pile, Disk disk) {
        if (!mapDiskToIcon.containsKey(disk)) {
            Color color = chooseColor(disk);
            DiskIcon icon = new DiskIcon(disk.getSize(), color);
            mapDiskToIcon.put(disk,icon);
            fixLocation(icon, pile);
            validate();
            repaint();
            return;
        }
        DiskIcon icon = (DiskIcon)mapDiskToIcon.get(disk);
        visualizeRobotPlaceDisk(icon, pile);
    }
    
    public void visualizeRemoveDisk(Pile pile, Disk disk) {
        int index = pileIndex(pile);
        visualizeRobotTakeDisk(index);
    }
    
    private void fixLocation(Component icon, Pile pile) {
        int index = pileIndex(pile);
        int xm = getPileHorizontalPosition(index);
        int bottom;
        if (pile.isEmpty()) {
            bottom = ground;
        }
        else {
            Disk topDisk = pile.topDisk();
            DiskIcon topDiskIcon = (DiskIcon)mapDiskToIcon.get(topDisk);
            bottom = topDiskIcon.getLocation().y-1;
        }
        int width = icon.getSize().width;
        icon.setLocation(xm-width/2, bottom-icon.getSize().height);
        add(icon);
        icon.repaint();
    }
    
    private int pileIndex(Pile pile) {
        for (int i=0; i<piles.size(); i++) {
            if (pile == piles.elementAt(i)) {
                return i;
            }
        }
        return -1; // this cannot happen
    }

    public void visualizeRobotTakeDisk(int pileIndex) {
        Disk disk = ((Pile)piles.elementAt(pileIndex)).topDisk();
        DiskIcon icon = (DiskIcon)mapDiskToIcon.get(disk);

        // Move the arm just above the pile
        Point origin = robot.getHeadLocation();
        Point target = new Point(icon.getCenter().x, ground-120);
        RobotArmMotionClip moveAboveChip =
            new RobotArmMotionClip(this,robot,origin,target,30);
            
        // Open robot hand
        RobotFingerMovementClip openHand =
            new RobotFingerMovementClip(this,robot,
                icon.getSize().width+6,10);
                
        // Go down to the disk        
        origin = new Point(target);
        target = new Point(icon.getCenter());
        target.translate(0,icon.getSize().height/2);
        RobotArmMotionClip goDown =
            new RobotArmMotionClip(this,robot,origin,target,30);
        RobotArmMotionClip goUp =
            new RobotArmMotionClip(this,robot,target,origin,30);
        goUp.setLoad(icon);

        AnimationClip[] clips =
            {moveAboveChip,openHand,goDown,goUp};
        AnimationClip clip =
            new AnimationClipSequence(clips);
        clip.setPreferredFrameDelay(35);
        clip.play(true);
    }
    
    private void visualizeRobotPlaceDisk(DiskIcon icon, Pile pile) {
        int pileIndex = pileIndex(pile);
        int x = getPileHorizontalPosition(pileIndex)-icon.getSize().width/2;
        int y;
        if (pile.isEmpty()) {
            y = ground;
        }
        else {
            DiskIcon topIcon = (DiskIcon)
                mapDiskToIcon.get(pile.topDisk());
            y = topIcon.getLocation().y;
        }
        Point origin = icon.getCenter();
        Point target = new Point(x,ground-120);
        
        RobotArmMotionClip moveAbovePile =
            new RobotArmMotionClip(this,robot,origin,target,30);
        moveAbovePile.setLoad(icon);
        
        RobotFingerMovementClip openWide =
            new RobotFingerMovementClip(this,robot,
                icon.getSize().width+6,
                icon.getSize().width+12,10);
        AnimationClip closeHand =
            new RobotFingerMovementClip(this,robot,
                icon.getSize().width+12,
                20,10);
        origin = new Point(target);
        target = new Point(x,y-icon.getSize().height-1);
        
        AnimationClip fall =
            new FreeFallClip(icon,origin,target,35);
        AnimationClip parallel =
            new AnimationClipUnion(closeHand,fall);
        AnimationClip drop =
            new AnimationClipSequence(openWide,parallel);
        AnimationClip[] clips =
            {moveAbovePile,parallel};
        AnimationClip clip =
            new AnimationClipSequence(clips);
        clip.setPreferredFrameDelay(35);
        clip.play(true);
        validate();
        repaint();
        try {
            Thread.sleep(50);
        } catch (InterruptedException ie) {}
    }

    public void paint(Graphics g) {
        int n = piles.size();
        for (int i=0; i<n; i++) {
            int x=getPileHorizontalPosition(i);
            g.setColor(Color.black);
            for (int j=-2; j<=2; j++) {
                g.drawLine(x+j,ground,x+j,ground-100);
            }
        }
        int width = getSize().width-rightMargin-leftMargin;
        g.setColor(Color.yellow);
        g.fillRect(leftMargin,ground,width,30);
        g.setColor(Color.black);
        g.drawRect(leftMargin,ground,width,30);
        robot.draw(g);
        super.paint(g);
    }
}