/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;

public class RobotArmFigure extends Figure {

    private static final int BASE_JOINT = 0;
    private static final int MAJOR_JOINT = 1;
    private static final int MINOR_JOINT = 2;
    private static final int HEAD_JOINT = 3;
    private static final int FINGER1_JOINT = 4;
    private static final int FINGER2_JOINT = 5;
    private static final int FINGERTIP1_JOINT = 6;
    private static final int FINGERTIP2_JOINT = 7;

    private RobotArmJoint[] joints;

    RobotArmJoint base;
    RobotArmJoint major;
    RobotArmJoint minor;
    RobotArmJoint head;
    RobotArmJoint finger1;
    RobotArmJoint finger2;
    RobotArmJoint fingertip1;
    RobotArmJoint fingertip2;

    int fingerSpacing = 20;

    private int thickness;
    private int baseSize;

    /**
     * Creates a new RobotArmFigure.
     */
    public RobotArmFigure(int baseSize,int thickness) {
        this.thickness = thickness;
        this.baseSize = baseSize;

        joints = new RobotArmJoint[8];
        double ratios[] =          {0.2,1.0,0.7,0.1,0.1,0.1,0.1,0.1};
        double thicknessRatios[] = {1.0,1.0,1.0,0.3,0.3,0.3,0.3,0.3};
        for (int i=0; i<joints.length; i++) {
            joints[i] = new RobotArmJoint(
                (int)(baseSize*ratios[i]),(int)(thickness*thicknessRatios[i]));
        }
        base  = joints[BASE_JOINT];
        major = joints[MAJOR_JOINT];
        minor = joints[MINOR_JOINT];
        head  = joints[HEAD_JOINT];
        finger1 = joints[FINGER1_JOINT];
        finger2 = joints[FINGER2_JOINT];
        fingertip1 = joints[FINGERTIP1_JOINT];
        fingertip2 = joints[FINGERTIP2_JOINT];
    }

    public Point getHeadLocation() {
        Point p = head.getEndPoint();
        return new Point(p.x,p.y+50); // temp
    }

    public void setHeadLocation(Point location) {
        int headSize = head.getSize()+
            (int)(finger1.getSize()*0.7)+fingertip1.getSize();

        Point p = head.getEndPoint();
        double dx = location.x-x;
        double dy = y-base.getSize()-location.y+headSize;
        double d = Math.sqrt(dx*dx+dy*dy);

        double a = major.getSize();
        double b = minor.getSize();

        double gamma = Math.acos((a*a+b*b-d*d)/(2*a*b));
        double alpha = Math.asin(b*Math.sin(gamma)/d);
        double delta = Math.atan(Math.abs(dy/dx));

        double teta1,teta2;
        if (dx<0) {
            teta1 = Math.PI-delta-alpha;
            teta2 = -alpha-delta-gamma;
        } else {
            teta1 = alpha+delta;
            teta2 = -(Math.PI-alpha-delta-gamma);
        }
        setJointsAngles(teta1,teta2);
    }

    public void setJointsAngles(double teta1,double teta2) {
        base.setLocation(x,y);
        base.setAngleDegrees(90);
        Point p = base.getEndPoint();
        major.setLocation(p.x,p.y);

        major.setAngleRadians(teta1);
        p = major.getEndPoint();
        minor.setLocation(p.x,p.y);
        minor.setAngleRadians(teta2);
        p = minor.getEndPoint();
        head.setLocation(p.x,p.y);
        head.setAngleDegrees(270);
        p = head.getEndPoint();
        finger1.setLocation(p.x,p.y);
        finger2.setLocation(p.x,p.y);

        setFingerSpacing(fingerSpacing);
        /*
        finger1.setAngleDegrees(270-25);
        finger2.setAngleDegrees(270+25);
        p = finger1.getEndPoint();
        fingertip1.setLocation(p.x,p.y);
        fingertip1.setAngleDegrees(270);
        p = finger2.getEndPoint();
        fingertip2.setLocation(p.x,p.y);
        fingertip2.setAngleDegrees(270);
        */
    }

    public int getFingerSpacing() {
        int width = (int)(finger1.getSize()*
            Math.abs(Math.cos(finger1.getAngleRadians()))*2);
        return width;
    }

    public void setFingerSpacing(int spacing) {
        fingerSpacing = spacing;
        double a=spacing/2.0;
        double teta = Math.asin(a/finger1.getSize());
        finger1.setAngleRadians(1.5*Math.PI-teta);
        finger2.setAngleRadians(1.5*Math.PI+teta);
        Point p = finger1.getEndPoint();
        fingertip1.setLocation(p.x,p.y);
        fingertip1.setAngleDegrees(270);
        p = finger2.getEndPoint();
        fingertip2.setLocation(p.x,p.y);
        fingertip2.setAngleDegrees(270);
    }

    public void draw(Graphics g) {
        for (int i=0; i<joints.length; i++) {
            joints[i].draw(g);
        }
        for (int i=0; i<joints.length; i++) {
            Point p = joints[i].getEndPoint();
            int r = joints[i].getThickness()/2;
            g.setColor(Color.lightGray);
            g.fillOval(p.x-r,p.y-r,2*r,2*r);
            g.setColor(Color.darkGray);
            g.drawOval(p.x-r,p.y-r,2*r,2*r);
            g.fillOval(p.x-r/2,p.y-r/2,r,r);
            g.setColor(Color.lightGray.darker());
            g.fillRect(x-40,y-30,80,60);

        }
    }
}