/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.awt.event.*;

/**
 * A window that can be used to display textual
 * output to the user.
 *
 *
 * @author Shmulik London
 */
public class OutputWindow extends Frame {

    // Margins from the upper and lower corners of the screen
    private static final int TOP_MARGIN = 20;
    private static final int BOTTOM_MARGIN = 70;
    private static final float HEIGHT_WIDTH_RATIO = 1.3F;

    // The text area used to display the text
    private TextArea display;

    /**
     * Constructs a new OutputWindow.
     */
    public OutputWindow() {
        this("Output");
    }

    /**
     * Constructs a new OutputWindow.
     * @param title The title for the window.
     */
    public OutputWindow(String title) {
        super(title);

        this.display = new TextArea();
        add(display, BorderLayout.CENTER);
        display.setFont(new Font("Monospaced",Font.BOLD,20));
        display.setBackground(Color.white);
        display.setForeground(new Color(0,0,150));

        Panel bottomPanel = new Panel();
        Button exitButton = new Button("Exit");
        exitButton.setFont(new Font("TimesRoman",Font.BOLD,14));
        bottomPanel.add(exitButton);
        add(bottomPanel, BorderLayout.SOUTH);

        Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();
        int height = screenSize.height-TOP_MARGIN-BOTTOM_MARGIN;
        int width = (int)(height/HEIGHT_WIDTH_RATIO);
        setSize(width, height);

        int left = (screenSize.width-getSize().width)/2;
        int top = TOP_MARGIN;
        setLocation(left,top);

        ClosingHandler handler = new ClosingHandler();
        addWindowListener(handler);
        exitButton.addActionListener(handler);

        setVisible(true);
    }

    // A listener class to enable the closing of the window
    // using its default closing button
    class ClosingHandler extends WindowAdapter
        implements ActionListener {

        /**
         * Called when the default closing button of the
         * window is pressed.
         */
        public void windowClosing(WindowEvent event) {
            System.exit(0);
        }

        /**
         * Notifies this listener that an action has been
         * performed.
         */
        public void actionPerformed(ActionEvent event) {
            System.exit(0);
        }
    }

    /**
     * Clear the display of the window.
     */
    public void clear() {
        display.setText("");
    }

    /**
     * Prints a line of text.
     */
    public void println(String line) {
        print(line+"\n");
    }

    /**
     * Prints an empty line.
     */
    public void println() {
        print("\n");
    }

    /**
     * Prints an numerical value and goes to the next line
     * of text.
     */
    public void println(double a) {
        println(String.valueOf(a));
    }

    /**
     * Prints an object description and goes to the next line
     * of text.
     */
    public void println(Object o) {
        println(o.toString());
    }

    /**
     * Prints an integer value to the output and goes to the next line
     * of text.
     */
    public void println(int a) {
        println(String.valueOf(a));
    }

    /**
     * Prints a string to the output.
     */
    public void print(String text) {
        display.append(text);
    }

    /**
     * Prints a numerical value to the output.
     */
    public void print(double a) {
        print(String.valueOf(a));
    }

    /**
     * Prints an integer value to the output.
     */
    public void print(int a) {
        print(String.valueOf(a));
    }

    /**
     * Prints the description of an object to the output.
     */
    public void print(Object o) {
        print(o.toString());
    }

    /**
     * Popup a message to the user.
     */
    public void showMessage(String message) {
        MessageDialog msgBox = new MessageDialog(this, message);
        Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();
        Dimension size = msgBox.getPreferredSize();
        msgBox.setSize(size.width,size.height);
        int left = (screenSize.width-size.width)/2;
        int top = (screenSize.height-size.height)/2;
        msgBox.setLocation(left,top);
        msgBox.setVisible(true);
    }

    static class MessageDialog extends Dialog {

        private static final int DEFAULT_HEIGHT = 140;

        private String message;
        private Button okButton;

        public MessageDialog(Frame parent, String message) {
            super(parent,true);
            this.message = message;
            setBackground(Color.white);
            setForeground(Color.black);
            setFont(new Font("Dialog",Font.BOLD,18));
            setLayout(new BorderLayout(20,30));
            Label label = new Label(message, Label.CENTER);
            okButton = new Button("OK");
            Panel okPanel = new Panel();
            okPanel.add(okButton);

            add(label, BorderLayout.CENTER);
            add(okPanel, BorderLayout.SOUTH);

            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    dispose();
                }
            });
        }

        /**
         * Returns the preferred size of this message box.
         */
        public Dimension getPreferredSize() {
            FontMetrics metrics = getFontMetrics(getFont());
            int width = (int)(metrics.stringWidth(message)*1.1+50);
            int height = DEFAULT_HEIGHT;
            return new Dimension(width,height);
        }
    }

}

