/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * A quardic function is a real function of the form
 * f(x)=ax<SUP><FONT size=-2>2</FONT></SUP>+bx+c.
 * This implementation assumes that the first coefficient
 * (the a) must be non-zero.
 *
 * @author Shmulik London
 */
public class QuadricFunction {

    // The coefficients of the function
    private double a,b,c;

    /**
     * Constructs a QuadricFunction with given coefficients.
     * @param a,b,c The coefficients of the function.
     * @exception IllegalArgumentException if the first 
     * coefficient of the function is zero, or one of the
     * coefficients is not a finite real number.
     */
    public QuadricFunction(double a, double b, double c) {
        if (a==0.0 || Double.isNaN(a) || Double.isNaN(b) ||
            Double.isNaN(c) || Double.isInfinite(a) ||
            Double.isInfinite(b) || Double.isInfinite(c)) {
           throw new IllegalArgumentException(
               "The coefficient of the function must be finite real "+
               "numbers with the first coefficient non-zero");
        }
        this.a = a;
        this.b = b;
        this.c = c;
    }

    /**
     * Return the value of the function at a given point.
     * @param x The point at which the function is evaluated.
     * @return The value of the function at x.
     */
    public double valueAt(double x) {
        return a*x*x+b*x+c;
    }

    /**
     * Returns the coefficient of x<SUP><FONT size=-2>2</FONT></SUP>.
     * @return The coefficient of x<SUP><FONT size=-2>2</FONT></SUP>.
     */
    public double getA() {
        return a;
    }

    /**
     * Returns the coefficient of x.
     * @return The coefficient of x.
     */
    public double getB() {
        return b;
    }

    /**
     * Returns the lowest degree coefficient.
     */
    public double getC() {
        return c;
    }

    /**
     * Return the number of roots of this quadric function
     * (the number of times this function crosses the x axis).
     * @return The number of roots of this quadric function.
     */
    public int getNumberOfRoots() {
        return (determinant()>0.0) ? 2 : (determinant()==0.0 ? 1 : 0);
    }

    /**
     * Returns the low root of this quadric function if one
     * exists. Returns the single root if there is only one
     * root.
     * @return The low root of this quadric function.
     */
    public double getLowRoot() {
        return (-b - Math.sqrt(determinant()))/(2*a);
    }

    /**
     * Returns the high root of this quadric function if one
     * exists. Returns the single root if there is only one
     * root.
     * @return The high root of this quadric function.
     */
    public double getHighRoot() {
        return (-b + Math.sqrt(determinant()))/(2*a);
    }

    /**
     * Returns the maximal value of this function.
     * If a>0 the maximal value is positive infinity.
     * @return The maximal value of this function.
     */
    public double getMaximalValue() {
        return (a>0.0) ? Double.POSITIVE_INFINITY : valueAt(getGlobalMaxima());
    }

    /**
     * Returns the minimal value of this function. If a<0
     * the minimal value is negative infinity.
     * @return The minimal value of this function.
     */
    public double getMinimalValue() {
        return (a<0.0) ? Double.NEGATIVE_INFINITY : valueAt(getGlobalMinima());
    }

    /**
     * Return the global maxima of this function - the point
     * where the value of the function is maximal. If a>0 returns
     * positive infinity.
     * @return The point where the value fo this function is maximal.
     */
    public double getGlobalMaxima() {
        return (a<0) ? -b/(2*a) : Double.POSITIVE_INFINITY;
    }

    /**
     * Return the global minima of this function - the point
     * were the value of the function is minimal. If a>0 returns
     * positive infinity.
     */
    public double getGlobalMinima() {
        return (a>0) ? -b/(2*a) : Double.POSITIVE_INFINITY;
    }

    // computes the determinant of a*x*x+b*x+c
    private double determinant() {
        return b*b-4*a*c;
    }

    /**
     * Return a quadric function which is the sum of this
     * function and a given function.
     * @param f A given quadric function.
     */
    public QuadricFunction sum(QuadricFunction f) {
        return new QuadricFunction(a+f.a, b+f.b, c+f.c);
    }

    /**
     * Returns a string description of this function
     * in the format ax^2+bx+c.
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append(formatTerm(a, true, 2));
        buffer.append(formatTerm(b, false, 1));
        buffer.append(formatTerm(c, false, 0));
        return buffer.toString();
    }
    
    // Formats a single term of the polynom 
    // value - the value of the coefficient
    // isFirst - is this the first coefficient (if so don't preceed with
    //     '-' or '+')
    // power - what power of x is this coefficient multiplied by (if 0
    //     treat 1.0 and -1.0 differently)
    private String formatTerm(double value, boolean isFirst, int power) {
        if (value == 0.0) {
            return "";
        }
        StringBuffer representation = new StringBuffer();
        if (!isFirst && value>0.0) {
            representation.append('+');
        }
        if (value==-1.0) {
            representation.append('-');
        }
        if (Math.abs(value)!=1.0 || power==0) {
            representation.append(value);
        }
        if (power>0) {
            if (power>1) {
                representation.append("x^"+power);
            }
            else {
                representation.append('x');
            }   
        }
        return representation.toString();
    }
}