/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.util.Enumeration;
import java.util.NoSuchElementException;

/**
 * A permutation. A permutation is a one-to-one mapping
 * from {1..n} onto {1..n}.
 * A permutation is immutable.
 *
 * @author Shmulik London
 */
public class Permutation implements java.io.Serializable {

    // mapping[i] represent the mapping of i under this permutation
    private int[] mapping;

    /**
     * Constructs a permutation.
     * @param mapping Defines the permutation.
     * <TT>mapping</TT> is assumed to contain a list of all
     * integers in the range <TT>1,..,mapping.length</TT>.
     * <TT>mapping[i]</TT>
     * denotes p(i+1) where p is the permutation.
     */
    public Permutation(int[] mapping) {
        this.mapping = new int[mapping.length];
        System.arraycopy(mapping, 0, this.mapping, 0, mapping.length);
    }
    
    /**
     * Returns the image of a given number under this permutation.
     * @throws IllegalArgumentException The argument is not in the
     * domain of this permutation.
     */
    public int map(int a) {
        try {
            return mapping[a-1];
        } catch (Exception e) {
            throw new IllegalArgumentException();
        }
    }
    
    /**
     * Return the order of the permutation. If the permutation
     * is from {1..n} to {1..n} the order is n.
     * @return The order of the permutation.
     */
    public int order() {
        return mapping.length;
    }

    /**
     * Arranges a given array according to this permutation.
     * @param source The array to be ordered
     * @param target The arranged array.
     */
    public void arrange(Object[] source, Object[] target) {
        if (source.length != mapping.length ||
            target.length != mapping.length) {
            throw new IllegalArgumentException();
        }
        int n = source.length;
        for (int i=0; i<n; i++) {
            target[mapping[i]]=source[i];
        }
    }

    /**
     * Returns the composite permutation resulting from
     * multiplying this permutation by another permutation.
     * @param p a permutation
     * @returns g(p) where g is this permutation
     */
    public Permutation multiply(Permutation p) {
        int[] compositeMapping = new int[mapping.length];
        for (int i=0; i<mapping.length; i++)
            compositeMapping[i]=mapping[p.mapping[i]-1];
        return new Permutation(compositeMapping);
    }

    /**
     * Tests if this permutation equals to another.
     * @param o The candidate for equality.
     */
    public boolean equals(Object o) {
        try {
            Permutation p = (Permutation) o;
            if (mapping.length!=p.mapping.length) {
                return false;
            }
            for (int i=0; i<mapping.length; i++) {
                if (mapping[i]!=p.mapping[i]) {
                    return false;
                }
            }
            return true;
        } catch (ClassCastException cce) {
            return false;
        }
    }

    /**
     * Returns a hash value for this permutation.
     * @return A hash value for this permutation.
     */
    public int hashCode() {
        // TODO: improve the hashing method
        // use the algorithm of Knuth
        int h=0;
        for (int i=0; i<mapping.length; i++) {
            h = (h*37)+mapping[i]+1;
        }
        return h;
    }

    /**
     * Returns the list of all permutations of {1..n}.
     * @return The list of all permutations of {1..n}.
     */
    public static Enumeration allPermutations(int n) {
        return new PermutationFactory(n);
    }

    /**
     * Returns a random permutation of {1..n} choosen
     * with uniform distribution over the set of permutations
     * on {1..n}.
     * @return A random permutation of {1..n}.
     */
    public static Permutation randomPermutation(int n) {
        return PermutationFactory.randomPermutation(n);
    }

    /**
     * Returns a String representation of this permutation
     * in the format "p(1),p(2),...,p(n)".
     * @return a String representation of this permutation.
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        for (int i=0; i<mapping.length; i++) {
            buffer.append(String.valueOf(mapping[i]+1));
            if (i<mapping.length-1)
                buffer.append(",");
        }
        return buffer.toString();
    }

    /**
     * Constructs the identity permutation on {1..n}.
     * The identity permutation on {1..n} is defined 
     * by p:{1..n}->{1..n} 1<=i<=n p(i)=i.
     * @returns The identity permutation on {1..n}.
     */
    public static Permutation identityPermutation(int n) {
        int[] mapping = new int[n];
        for (int i=0; i<n; i++) {
            mapping[i]=i+1;
        }
        return new Permutation(mapping);
    }

    // The inner class PermutationFactory is a factory for permutations.
    // It can produce a list of all permutations of a given order.
    static class PermutationFactory implements Enumeration {
        
        // The order of the permutations
        private int n;
                
        // The order of selecting from {1..n}.
        // counter advences from 0,0,..,0 like a dinamometer counter.
        private int[] counter;
        
        // True if this factory hasn't created yet a permutation
        private boolean firstTime=true;

        /**
         * Constructs an enumeration of all permutations of the objects in the array 'objects'
         */
        public PermutationFactory(int n) {
            this.n=n;
            //indexes=new int[n];
            counter=new int[n];
        }

        // ------------------------------ implementation -----------------------------------------------
        // we generate a permutation by selection, we first pick up the first item, then
        // pick up a second one from those that left and so on.
        // we have a counter that tells us the order of the selection for each permutation.
        // For example if we have 4 items: items={a,b,c,d}, and currently counter={2,1,1,0},
        // we will generate the next permutation by choosing first 'c' (the third item - 2 from 0),
        // then 'b' (the second (1) left of a,b,d), then 'd' and then 'a'.
        // after each permutation we advance the counter, were it's first place can go over 0..n-1, its
        // second trough 0..n-2 etc.
        // the indexes array keeps trace of which is the first, second,.. item in the remaining list.

        private void advanceCounter() {
            int i=n-1; // the last place is always 0, as there is only one choice for the last item.
            do {
                i--;
                if (i<0) {
                    throw new NoSuchElementException();
                }
                counter[i]=(counter[i]+1)%(n-i); // for the i's place there are n-i remining choices
            } while (counter[i]==0);
        }

        /**
         * Checks is there are more permutations to produce
         * @return true iff there is more permutations to be produces
         */
        public boolean hasMoreElements() {
            for (int i=n-2; i>=0; i--)
                if (counter[i]!=n-i-1)
                    return true;
            return false;
        }

        /**
         * Produces the next premutation
         * @return The next permutation
         */
        public Permutation nextPermutation() {
            if (!firstTime) {
                advanceCounter();
            }
            else {
                firstTime=false;
            }
            int[] permutation = new int[n];
            
            // The numbers {1,2,...,n} which will be arranged
            int[] indexes = new int[n];
            for (int i=0; i<n; i++) {
                indexes[i]=i;
            }
            for (int i=0; i<n; i++) {
                int choice=counter[i];
                permutation[i]=indexes[choice];
                for (int j=choice; j<n-i-1; j++) {
                    indexes[j]=indexes[j+1];
                }
            }
            return new Permutation(permutation);
        }

        /**
         * Returns the next permutation.
         * @return The next permutation.
         */
        public Object nextElement() {
            return nextPermutation();
        }

        /**
         * Returns a randomly choosen permutation over {1..n}.
         * @return A randomly choosen permutation over {1..n}.
         */
        public static Permutation randomPermutation(int n) {
            PermutationFactory p = new PermutationFactory(n);
            int[] counter = new int[n];
            for(int i=0; i<n-1; i++) {
                counter[i]=(int)(Math.random()*(n-i));
            }
            p.counter=counter;
            p.firstTime=true;
            return p.nextPermutation();
        }
    }

}

