/*
 * (c) Copyright 1998 by Shmulik London (londons@gmail.com). All rights reserved.
 *
 * Redistribution and use of this source code, with or without modification,
 * are permitted for educational non-commercial use provided that it retains
 * the above copyright notice and the following disclaimer.
 *
 * This source code was developed to support the course 'Introduction to Computer Science'
 * given in the Interdisciplinary Center Herzelia in 1998,1999 by Shmulik London.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. THE AUTHOR OF THIS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/**
 * A quardic function is a real function of the form
 * f(x)=ax<SUP><FONT size=-2>2</FONT></SUP>+bx+c.
 *
 * @author Shmulik London
 */
public class QuadricFunction {

    // The coefficients of the function
    private double a,b,c;

    /**
     * Constructs a QuadricFunction with given coefficients.
     * @param a,b,c The coefficients of the function.
     */
    public QuadricFunction(double a, double b, double c) {
        this.a = a;
        this.b = b;
        this.c = c;
    }

    /**
     * Return the value of the function at a given point.
     * @param x The point at which the function is evaluated.
     * @return The value of the function at x.
     */
    public double valueAt(double x) {
        double value = a*x*x+b*x+c;
        return value;
    }

    /**
     * Returns the coefficient of x<SUP><FONT size=-2>2</FONT></SUP>.
     */
    public double getA() {
        return a;
    }

    /**
     * Returns the coefficient of x.
     */
    public double getB() {
        return b;
    }

    /**
     * Returns the lowest degree coefficient.
     */
    public double getC() {
        return c;
    }

    /**
     * Return the number of roots of this quadric function
     * (the number of times this function crosses the x axis).
     */
    public int getNumberOfRoots() {
        double determinant = determinant();
        if (determinant<0) {
            return 0;
        }
        if (determinant==0) {
            return 1;
        }
        return 2;
    }

    /**
     * Returns the low root of this quadric function if one
     * exists.
     */
    public double getLowRoot() {
        if (a==0.0) {
            if (b==0) {
                return (c==0 ? 0.0 : Double.NaN);
            }
            return -b/c;
        }
        return (-b - Math.sqrt(determinant()))/(2*a);
    }

    /**
     * Returns the high root of this quadric function if one
     * exists.
     */
    public double getHighRoot() {
        if (a==0.0) {
            if (b==0) {
                return (c==0 ? 0.0 : Double.NaN);
            }
            return -b/c;
        }
        return (-b + Math.sqrt(determinant()))/(2*a);
    }

    /**
     * Returns the maximal value of this function.
     */
    public double getMaximalValue() {
        if (a>0 || (a==0 && b!=0)) {
            return Double.POSITIVE_INFINITY;
        }
        return valueAt(getGlobalMaxima());
    }

    /**
     * Returns the minimal value of this function.
     */
    public double getMinimalValue() {
        if (a<0 || (a==0 && b!=0)) {
            return Double.NEGATIVE_INFINITY;
        }
        return valueAt(getGlobalMinima());
    }

    /**
     * Return the global maxima of this function - the point
     * were the value of the function is maximal.
     */
    public double getGlobalMaxima() {
        if (a<0) {
            return -b/a;
        }
        if (a==0 && b==0) { // any point is a global maxima
            return 0;
        }
        return Double.POSITIVE_INFINITY;
    }

    /**
     * Return the global minima of this function - the point
     * were the value of the function is minimal.
     */
    public double getGlobalMinima() {
        if (a>0) {
            return -b/a;
        }
        if (a==0 && b==0) { // any point is a global minima
            return 0;
        }
        return Double.NEGATIVE_INFINITY;
    }

    // computes the determinant of a*x*x+b*x+c
    private double determinant() {
        return b*b-4*a*c;
    }

    /**
     * Return a quadric function which is the sum of this
     * function and a given function.
     * @param function A given quadric function.
     */
    public QuadricFunction sum(QuadricFunction function) {
        double sumA = a + function.a;
        double sumB = b + function.b;
        double sumC = c + function.c;
        return new QuadricFunction(sumA, sumB, sumC);
    }

    /**
     * Returns a string description of this function
     * in the format ax^2+bx+c.
     */
    public String toString() {
        return a+"x^2+"+b+"x+"+c;
    }
}